@@ -37,6 +37,9 @@ gboolean nautilus_file_name_matches_backup_pattern   (const char *name_or_relati
 char *   nautilus_get_user_directory                 (void);
 char *   nautilus_get_desktop_directory              (void);
 char *   nautilus_get_desktop_directory_uri          (void);
+gboolean nautilus_is_desktop_directory_file_escaped  (char *escaped_dirname,
+						      char *escaped_filename);
+gboolean nautilus_is_desktop_directory_escaped       (char *escaped_dir);
 char *   nautilus_get_gmc_desktop_directory          (void);
 char *   nautilus_get_pixmap_directory               (void);
 
