@@ -1198,6 +1198,8 @@ create_normal_cache_icon (const char *icon,
 	
 	cache_icon = cache_icon_new (pixbuf, icon_data);
 	cache_icon->mtime = mtime;
+
+	g_object_unref (pixbuf);
 	
 	return cache_icon;
 }
@@ -1423,7 +1425,7 @@ nautilus_get_icon_size_for_zoom_level (NautilusZoomLevel zoom_level)
 	case NAUTILUS_ZOOM_LEVEL_LARGEST:
 		return NAUTILUS_ICON_SIZE_LARGEST;
 	}
-	g_return_val_if_fail (FALSE, NAUTILUS_ICON_SIZE_STANDARD);
+	g_return_val_if_reached (NAUTILUS_ICON_SIZE_STANDARD);
 }
 
 /* Convenience cover for nautilus_icon_factory_get_icon_for_file
